import React, {Component, PropTypes} from 'react'
import Bars from './bars'
import YAxis, {YAxisDefault} from '../YAxis'
import XAxis, {XAxisDefault} from '../XAxis'

import { getColorScale } from '../utils/colorScale'

const scale = require('d3-scale');
const _ = require('d3-array');

class BarChart extends Component {

  static propTypes = {
    data: PropTypes.array.isRequired,
    background: PropTypes.string,
    bandsPaddingInner: PropTypes.number,
    bandsPaddingOuter: PropTypes.number,
    chartClassName: PropTypes.string,
    color: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.array
    ]),
    height: PropTypes.number,
    padding: PropTypes.object,
    width: PropTypes.number,
    xAxis: PropTypes.object,
    yAxis: PropTypes.object
  };

  static defaultProps = {
    background: '#eee',
    bandsPaddingInner: 0.2,
    bandsPaddingOuter: 0.1,
    chartClassName: 'r3-barchart',
    color: '#009688',
    height: 300,
    width: 400
  };

  constructor(props) {
    super(props);
    this.conf = {
      padding: {
        bottom: 10,
        left: 10,
        right: 15,
        top: 20
      },
      xAxis: Object.assign({}, XAxisDefault, {
        className: 'r3-barchart-xaxis',
        height: 30
      }),
      yAxis: Object.assign({}, YAxisDefault, {
        className: 'r3-barchart-yaxis',
        width: 35
      })
    }
  }

  /**
   * Вычисляем domain для шкалы y.
   * Можно задать минимальное и максимальное значение по шкале y,
   * иначе будет браться мин. и макс. из выборки.
   * Также к этим значениям прибавляетя 10%, для того чтобы минимальные значения
   * не лежали прямо на оси, а подписи к максимальным не выходили за пределы
   * @param {Array}  data -- данные графика
   * @param {Number} [min] -- минимальное значение по шкале y
   * @param {Number} [max] -- максимальное значение по шкале y
   * @return {Array} -- domain для шкалы x
   */
  getYDomain(data, min, max) {
    min = min || _.min(data, (v)=> v.value);
    max = max || _.max(data, (v)=> v.value);
    return [min + min * .1, max + max*.1]
  }

  /**
   * геттер для стилей графика (svg)
   * @return {Object} css
   */
  getStyle() {
    const {background} = this.props;
    return {
      background
    }
  }

  render() {

    const { bandsPaddingInner,
            bandsPaddingOuter,
            chartClassName,
            color,
            data,
            height,
            width } = this.props;

    const padding = Object.assign({}, this.conf.padding, this.props.padding);
    const xAxis = Object.assign({}, this.conf.xAxis, this.props.xAxis);
    const yAxis = Object.assign({}, this.conf.yAxis, this.props.yAxis);

    // если ось не показывается, то её высота / ширина равна 0
    xAxis.height = xAxis.disable ? 0 : xAxis.height;
    yAxis.width = yAxis.disable ? 0 : yAxis.width;

    // ширина и высота доступной области данных для графика
    const barsWidth = width - yAxis.width - padding.left - padding.right;
    const barsHeight = height - xAxis.height - padding.top - padding.bottom;

    const colorScale = getColorScale(color);

    const xScale = scale.scaleBand()
      .domain(data.map((v) => v.name))
      .rangeRound([0, barsWidth])
      .paddingInner(bandsPaddingInner)
      .paddingOuter(bandsPaddingOuter);

    const yScale = scale.scaleLinear()
      .domain(this.getYDomain(data))
      .range([0, barsHeight]);

    // получаем массив делений, в соответствии с заданным количеством,
    const yTicks = yScale.ticks(yAxis.tickCount);

    return (
      <svg className={chartClassName}
        height={height}
        style={this.getStyle()}
        width={width}>

        <g transform={`translate(${padding.left},${padding.top})`}>
          <Bars colorScale={colorScale}
            data={data}
            dx={yAxis.width}
            height={barsHeight}
            width={barsWidth}
            xScale={xScale}
            yScale={yScale}/>

          <XAxis className={xAxis.className}
            disable={xAxis.disable}
            dx={yAxis.width}
            dy={barsHeight}
            fontColor={xAxis.fontColor}
            fontSize={xAxis.fontSize}
            lineColor={xAxis.lineColor}
            lineWidth={xAxis.lineWidth}
            tickColor={xAxis.tickColor}
            ticks={xScale.domain()}
            tickWidth={xAxis.tickWidth}
            tickShift={xScale.bandwidth()/2}
            width={barsWidth}
            xScale={xScale}/>

          <YAxis className={yAxis.className}
            disable={yAxis.disable}
            dx={yAxis.width}
            fontColor={yAxis.fontColor}
            fontSize={yAxis.fontSize}
            height={barsHeight}
            lineColor={yAxis.lineColor}
            lineWidth={yAxis.lineWidth}
            ticks={yTicks}
            tickColor={yAxis.tickColor}
            tickWidth={yAxis.tickWidth}
            yScale={yScale}/>
        </g>

      </svg>
    );
  }

}

export default BarChart;
