import React, {Component, PropTypes} from 'react'
import Boxes from './boxes'
import YAxis, {YAxisDefault} from '../YAxis'
import XAxis, {XAxisDefault} from '../XAxis'

import { getColorScale } from '../utils/colorScale'

const scale = require('d3-scale');
const _ = require('d3-array');

class HeatMap extends Component {

  static propTypes = {
    data: PropTypes.array.isRequired,
    color: PropTypes.array.isRequired,
    background: PropTypes.string,
    margin: PropTypes.number,
    chartClassName: PropTypes.string,
    padding: PropTypes.object,
    size: PropTypes.number,
    xAxis: PropTypes.object,
    yAxis: PropTypes.object
  };

  static defaultProps = {
    background: '#eee',
    margin: 0,
    chartClassName: 'r3-heat-map',
    size: 14
  };

  constructor(props) {
    super(props);
    this.conf = {
      padding: {
        bottom: 15,
        left: 20,
        right: 115,
        top: 40
      },
      xAxis: Object.assign({}, XAxisDefault, {
        className: 'r3-heat-map-xaxis',
        height: 30
      }),
      yAxis: Object.assign({}, YAxisDefault, {
        className: 'r3-heat-map-yaxis',
        lineWidth: 0,
        modTick: 10,
        tickWidth: 0,
        width: 35
      })
    }
  }

  /**
   * геттер для стилей графика (svg)
   * @return {Object} css
   */
  getStyle() {
    const {background} = this.props;
    return {
      background
    }
  }

  getYDomain(data) {
    let res = [];
    data.forEach((item)=> {
      if (!~res.indexOf(item.name)) {
        res.push(item.name)
      }
    });
    return res
  }

  getYRange(data, size, margin) {
    return this.getYDomain(data).map((item, i)=>i * (size * (1 + margin)))
  }

  getXTicks(min, max, mod) {
    let res = [];
    for (let i = min; i < max; i++) {
      if (!(i % mod)) {
        res.push(i)
      }
    }
    return res
  }

  render() {

    const { background,
            margin,
            chartClassName,
            color,
            data,
            size } = this.props;

    if (data.length) {

      const padding = Object.assign({}, this.conf.padding, this.props.padding);
      const xAxis = Object.assign({}, this.conf.xAxis, this.props.xAxis);
      const yAxis = Object.assign({}, this.conf.yAxis, this.props.yAxis);

      // если ось не показывается, то её высота / ширина равна 0
      xAxis.height = xAxis.disable ? 0 : xAxis.height;
      yAxis.width = yAxis.disable ? 0 : yAxis.width;

      const xMin = _.min(data, (val) => val.x);
      const xMax = _.max(data, (val) => val.x);

      const boxesHeight = this.getYDomain(data).length * size * (1 + margin);
      const boxesWidth = (xMax - xMin + 1) * size * (1 + margin);

      const width = boxesWidth + yAxis.width + padding.left + padding.right;
      const height = boxesHeight + xAxis.height + padding.top + padding.bottom;

      const colorScale = getColorScale(color);

      const yScale = scale.scaleOrdinal()
        .domain(this.getYDomain(data))
        .range(this.getYRange(data, size, margin));

      // шкала необходима для YAxis, потому что ось
      // начинается не как всегда снизу, а сверху
      const yScaleReverse = scale.scaleOrdinal()
        .domain(this.getYDomain(data).reverse())
        .range(this.getYRange(data, size, margin));

      const xScale = function (x) {
        return (x - xMin) * (size + size * margin)
      };

      const xTicks = this.getXTicks(xMin, xMax, yAxis.modTick);
      const yTicks = yScale.domain();


      return (
        <svg className={chartClassName}
             height={height}
             style={this.getStyle()}
             width={width}>

          <g transform={`translate(${padding.left},${padding.top})`}>
            <Boxes background={background}
                   colorScale={colorScale}
                   data={data}
                   dx={yAxis.width}
                   width={boxesWidth}
                   size={size}
                   xScale={xScale}
                   yScale={yScale}
                   background={background}/>

            <XAxis className={xAxis.className}
                   disable={xAxis.disable}
                   dx={yAxis.width}
                   dy={boxesHeight}
                   fontColor={xAxis.fontColor}
                   fontSize={xAxis.fontSize}
                   lineColor={xAxis.lineColor}
                   lineWidth={xAxis.lineWidth}
                   tickColor={xAxis.tickColor}
                   ticks={xTicks}
                   tickShift={size/2}
                   tickWidth={xAxis.tickWidth}
                   width={boxesWidth}
                   xScale={xScale}/>

            <YAxis className={yAxis.className}
                   disable={yAxis.disable}
                   dx={yAxis.width}
                   fontColor={yAxis.fontColor}
                   fontSize={yAxis.fontSize}
                   height={boxesHeight}
                   lineColor={yAxis.lineColor}
                   lineWidth={yAxis.lineWidth}
                   ticks={yTicks}
                   tickColor={yAxis.tickColor}
                   tickShift={size*margin + size/2}
                   tickWidth={yAxis.tickWidth}
                   yScale={yScaleReverse}/>
          </g>

        </svg>
      );
    } else {
      return null
    }

  }

}

export default HeatMap;
