import React, {Component, PropTypes} from 'react'
import Point from './point'
import {animatePath, hidePath} from '../utils/svg'
import {deepEqual} from '../utils/object'

const shape = require('d3-shape');

class Line extends Component {

  static propTypes = {
    dx: PropTypes.number.isRequired,
    fill: PropTypes.string.isRequired,
    height: PropTypes.number.isRequired,
    lineWidth: PropTypes.number.isRequired,
    voronoi: PropTypes.array.isRequired,
    width: PropTypes.number.isRequired,
    xScale: PropTypes.func.isRequired,
    yScale: PropTypes.func.isRequired
  };

  /**
   * Обновляем компонент только когда данные диаграммы Вороного новые
   * @param nextProps
   * @return {boolean}
   */
  shouldComponentUpdate(nextProps) {
    return !deepEqual(this.props.voronoi, nextProps.voronoi)
  }

  /**
   * При инициализации компонента сначала скрываем линию,
   * далее после 300ms задержки (после того как все точки встали на свои места),
   * анимируем линию
   */
  componentDidMount() {
    hidePath(this.refs.line);
    setTimeout(()=> {
      animatePath(this.refs.line)
    }, 400)
  }

  /**
   * При перерендре компонента, если есть таймер то убиваем его, потом скрываем линию,
   * далее после 400ms задержки (после того как все точки встали на свои места),
   * анимируем линию
   */
  componentWillUpdate() {
    if (this.timer1) {
      clearTimeout(this.timer1)
    }

    hidePath(this.refs.line);
    this.timer1 = setTimeout(()=> {
      animatePath(this.refs.line)
    }, 400)
  }


  /**
   * Возвращает объект для атрибута style, для обертки над точками
   * @returns {{transform: *}}
   */
  getStyle() {
    const { dx } = this.props;
    return {transform: `translate(${dx}px,0px)`}
  }

  /**
   * Возвращает линию для графика в path
   * @returns {XML}
   */
  getLine() {
    let { fill, lineWidth, voronoi } = this.props;
    let _data = [];

    // при определенных входных данных массив voronoi (d3-voronoi)
    // может быть разряжен и быть осторожно с этим
    // например при построении линии (d3-shape) не допускается разряженный
    // массив в качестве входных данных
    voronoi.forEach((item)=> {
      _data.push([
        this.props.xScale(item.data[0]),
        this.props.height - this.props.yScale(item.data[1])
      ])
    });
    return <path ref="line" d={shape.line()(_data)} fill="transparent" stroke={fill} strokeWidth={lineWidth}/>
  }

  /**
   * Строем svg path, который является полигоном диаграммы Вороного.
   * + замыкаем его (строка "Z" в конце значения аттрибута d в path)
   * @param data массив точек, каждая из которых является вершиной многоугольника
   * @return {string} многоугольник в виде приемлемом для svg path
   */
  getPath(data) {
    const res = data.map((item)=> {
      return [
        this.props.xScale(item[0]),
        this.props.height - this.props.yScale(item[1])
      ]
    });
    return shape.line()(res) + 'Z'
  }

  render() {
    const { fill,
            height,
            voronoi,
            xScale,
            yScale,
            z } = this.props;

    // находим центр диаграммы, чтобы новые точки появлялись из нее
    // далее передаем это в компонент Point
    const xCenter = xScale.range()[1] / 2;
    const yCenter = yScale.range()[1] / 2;

    const points = voronoi.map((val, i) => {
      const _height = yScale(val.data[1]),
            path    = this.getPath(val),
            x       = xScale(val.data[0]),
            y       = height - _height;

      return (
        <Point fill={fill}
               key={i}
               path={path}
               value={val.data}
               x={x}
               xCenter={xCenter}
               yCenter={yCenter}
               y={y}
               z={z}/>
      )
    });

    return (
      <g style={this.getStyle()}>
        {this.getLine()}
        {points}
      </g>
    );
  }

}

export default Line;
