import React, {Component, PropTypes} from 'react'
import Points from './points'
import YAxis, {YAxisDefault} from '../YAxis'
import XAxis, {XAxisDefault} from '../XAxis'

import { getColorScale } from '../utils/colorScale'

const scale = require('d3-scale');
const _ = require('d3-array');
const voronoi = require('d3-voronoi');

class ScatterPlot extends Component {

  static propTypes = {
    data: PropTypes.array.isRequired,
    background: PropTypes.string,
    chartClassName: PropTypes.string,
    color: PropTypes.string,
    height: PropTypes.number,
    padding: PropTypes.object,
    width: PropTypes.number,
    xAxis: PropTypes.object,
    yAxis: PropTypes.object,
    z: PropTypes.oneOfType([
      PropTypes.number,
      PropTypes.array
    ])
  };

  static defaultProps = {
    background: '#eee',
    chartClassName: 'r3-scatter-plot',
    color: 'red',
    height: 300,
    width: 400,
    z: 3
  };

  constructor(props) {
    super(props);
    this.conf = {
      padding: {
        bottom: 10,
        left: 10,
        right: 15,
        top: 20
      },
      xAxis: Object.assign({}, XAxisDefault, {
        className: 'r3-scatter-plot-xaxis',
        domainDelta: .5,
        height: 30
      }),
      yAxis: Object.assign({}, YAxisDefault, {
        className: 'r3-scatter-plot-yaxis',
        domainDelta: .5,
        width: 35
      })
    }
  }

  /**
   * Вычисляем domain для шкал.
   * Можно задать минимальное и максимальное значение по шкалам,
   * иначе будет браться мин. и макс. из выборки.
   * Также к этим значениям прибавляетя delta, (по умолчанию 10%),
   * для того чтобы минимальные значения
   * не лежали прямо на оси, а подписи к максимальным не выходили за пределы
   * @param {'x'|'y'|'z'} axis -- ось
   * @param {Array}  data -- данные графика
   * @param {number} [delta] -- поправка к шкалам
   * @param {number} [min] -- минимальное значение по шкале
   * @param {number} [max] -- максимальное значение по шкале
   * @return {Array} -- domain для шкалы
   */
  getDomain(axis, data, delta=.1, min, max) {
    min = min || _.min(data, (v)=> v[axis]);
    max = max || _.max(data, (v)=> v[axis]);
    return [min - Math.abs(min * delta), max + Math.abs(max * delta)]
  }

  /**
   * геттер для оси z,
   * ось возвращается в соответсвии с переданами значениями в компонент
   * @return {function} шкала z
   */
  getZScale() {
    const { data, z } = this.props;
    if (typeof z === 'number') {
      return function(){
        return z
      }
    } else {
      return scale.scaleLinear()
        .domain(this.getDomain('z', data, 0))
        .range(z);
    }
  }

  /**
   * геттер для стилей графика (svg)
   * @return {Object} css
   */
  getStyle() {
    const {background} = this.props;
    return {
      background
    }
  }

  render() {

    const { chartClassName,
            color,
            data,
            height,
            width } = this.props;

    const padding = Object.assign({}, this.conf.padding, this.props.padding);
    const xAxis = Object.assign({}, this.conf.xAxis, this.props.xAxis);
    const yAxis = Object.assign({}, this.conf.yAxis, this.props.yAxis);

    // если ось не показывается, то её высота / ширина равна 0
    xAxis.height = xAxis.disable ? 0 : xAxis.height;
    yAxis.width = yAxis.disable ? 0 : yAxis.width;

    // ширина и высота доступной области данных для графика
    const graphWidth = width - yAxis.width - padding.left - padding.right;
    const graphHeight = height - xAxis.height - padding.top - padding.bottom;

    const colorScale = getColorScale(color);

    const xScale = scale.scaleLinear()
      .domain(this.getDomain('x', data, xAxis.domainDelta))
      .range([0, graphWidth]);

    const yScale = scale.scaleLinear()
      .domain(this.getDomain('y', data, yAxis.domainDelta))
      .range([0, graphHeight]);

    // шкала для линейных размеров точки
    // (например радиуса кругов, или стороны квадрата)
    const zScale = this.getZScale();

    // строим диаграмму Вороного для приятного эффекта ховера на точку
    const voronoiData = voronoi.voronoi();
    voronoiData.extent([
      [
        xScale.domain()[0],
        yScale.domain()[0]
      ],
      [
        xScale.domain()[1],
        yScale.domain()[1]
      ]
    ]);

    const point = data.map((item)=>{
      return [item.x, item.y];
    });

    // В возращенном массивы диаграммы voronoi нет информации о параметре z
    // поэтому вшиваем его
    let _voronoiData = voronoiData(point).polygons().map((item, i)=>{
      item.data.push(data[i].z);
      return item
    });

    // получаем значения отображаемые на осях -- засечки,
    const yTicks = yScale.ticks(yAxis.tickCount);
    const xTicks = xScale.ticks(xAxis.tickCount);

    return (
      <svg className={chartClassName}
           height={height}
           style={this.getStyle()}
           width={width}>

        <g transform={`translate(${padding.left},${padding.top})`}>
          <Points colorScale={colorScale}
                  dx={yAxis.width}
                  fill={color}
                  height={graphHeight}
                  voronoi={_voronoiData}
                  width={graphWidth}
                  xScale={xScale}
                  yScale={yScale}
                  zScale={zScale}/>

          <XAxis className={xAxis.className}
                 disable={xAxis.disable}
                 dx={yAxis.width}
                 dy={graphHeight}
                 fontColor={xAxis.fontColor}
                 fontSize={xAxis.fontSize}
                 lineColor={xAxis.lineColor}
                 lineWidth={xAxis.lineWidth}
                 tickColor={xAxis.tickColor}
                 ticks={xTicks}
                 tickWidth={xAxis.tickWidth}
                 width={graphWidth}
                 xScale={xScale}/>

          <YAxis className={yAxis.className}
                 disable={yAxis.disable}
                 dx={yAxis.width}
                 fontColor={yAxis.fontColor}
                 fontSize={yAxis.fontSize}
                 height={graphHeight}
                 lineColor={yAxis.lineColor}
                 lineWidth={yAxis.lineWidth}
                 ticks={yTicks}
                 tickColor={yAxis.tickColor}
                 tickWidth={yAxis.tickWidth}
                 yScale={yScale}/>
        </g>

      </svg>
    );
  }

}

export default ScatterPlot;
