import {getBuiltInClass} from './class'
const scale = require('d3-scale');

/**
 * Интерфейс который выдаем наружу.
 * Функция возвращает цветовую шкалу в зависимости от переданного параметра.
 * Если тип входного параметра строка, являющаяся цветом в hex, то возвращается шкала,
 * которая на любое значение возвращает переданный цвет.
 * Если передается "карта" с цветом то, возвращается соответсвующая линейная шкала
 * Формат передаваемой карты:
 * [
 *   {
 *     color:'#0000FF',
 *     value: 30
 *   },
 *   {
 *     color:'#135BA9',
 *     value: 20
 *   },
 *   {
 *     color:'#98cced',
 *     value: 0
 *   }
 * ]
 * @param {Object[]|string} color -- входные данные
 * @returns {Function} -- цветовая шкала
 */
export function getColorScale(color){
  switch (getBuiltInClass(color)) {
    case 'Array':
      return (scale.scaleLinear()
        .domain(getColorDomain(color))
        .range(getColorRange(color)));
    case 'String':
      return getDefaultColorScale(color)
  }
}

function getDefaultColorScale(color){
  return function() {
    return color
  }
}


/*
 * Сортируем входной массив с цветами по возрастанию ключа value
 * Так же есть возможность присваивать цвет максимальному / минимальному
 * значению из набора указав в поле value соотвественно 'max' / 'min'
 * @param {Array} color -- данные о сопоставлении цвета и значению
 * @param {Array} -- данные о сопоставлении цвета и значению по возрастанию
 */
function getSortColor(color) {
  return color.sort((a,b) => {
    if (typeof a.value === 'number' && typeof b.value === 'number') {
      return a.value - b.value
    } else if (a.value === 'min') {
      return -1
    } else if (b.value === 'min') {
      return 1
    } else if (a.value === 'max') {
      return 1
    } else if (b.value === 'max') {
      return -1
    }
  });
}
/*
 * Вычисляем range для цветовой шкалы
 * @param {Array} color -- данные о сопоставлении цвета и значению
 * @return {Array} -- range для цветовой шкалы
 */
function getColorRange(color) {
  return getSortColor(color).map((v) => v.color)
}

/*
 * Вычисляем domain для цветовой шкалы
 * @param {Array} color -- данные о сопоставлении цвета и значения
 * @param {number} min -- минимальное значение из входного диапазона
 * @param {number} max -- максимальное значение из входного диапазона
 * @return {Array} -- domain для цветовой шкалы
 */
function getColorDomain(color, min, max) {
  return getSortColor(color).map((v) => {
    switch(v.value) {
      case 'min':
        return min;
      case 'max':
        return max;
      default:
        return v.value
    }
  })
}
